@@ -124,7 +124,7 @@ void do_warn(SV* h, int rc, char* what) {
  **************************************************************************/
 
 int MyConnect(dbh_t *sock, char* host, char* port, char* user,
-	      char* password, char* dbname) {
+	      char* password, char* dbname, imp_dbh_t *imp_dbh) {
     int portNr;
 
     if (host && !*host) host = NULL;
@@ -163,6 +163,11 @@ int MyConnect(dbh_t *sock, char* host, char* port, char* user,
 	return result;
 #elif defined(MYSQL_VERSION_ID)  &&  (MYSQL_VERSION_ID >= 32200)
 	mysql_init(*sock);
+#if defined(MYSQL_VERSION_ID)  &&  (MYSQL_VERSION_ID >= 32203)
+	if (imp_dbh  &&  imp_dbh->compression) {
+	    mysql_options(*sock, MYSQL_OPT_COMPRESS, NULL);
+	}
+#endif
 	return mysql_real_connect(*sock, host, user, password, dbname,
 				  portNr, NULL, 0) ?
 	    TRUE : FALSE;
@@ -262,6 +267,14 @@ static int _MyLogin(imp_dbh_t *imp_dbh) {
     } else {
         dbname = NULL;
     }
+#ifdef DBD_MYSQL
+    if ((svp = hv_fetch(hv, "mysql_compression", 17, FALSE))  &&
+	*svp  &&  SvTRUE(*svp)) {
+        imp_dbh->compression = 1;
+    } else {
+        imp_dbh->compression = 0;
+    }
+#endif
 
     if (dbis->debug >= 2)
         fprintf(DBILOGFP,
@@ -278,11 +291,13 @@ static int _MyLogin(imp_dbh_t *imp_dbh) {
 #endif
 
 #if defined(DBD_MYSQL)  &&  (MYSQL_VERSION_ID >= 32200)
-    if (!MyConnect(&imp_dbh->svsock, host, port, user, password, dbname)) {
+    if (!MyConnect(&imp_dbh->svsock, host, port, user, password, dbname,
+		   imp_dbh)) {
         return FALSE;
     }
 #else
-    if (!MyConnect(&imp_dbh->svsock, host, port, user, password, NULL)) {
+    if (!MyConnect(&imp_dbh->svsock, host, port, user, password, NULL,
+		   imp_dbh)) {
         return FALSE;
     }
 
@@ -1187,6 +1202,56 @@ SV* dbd_st_FETCH_internal(SV* sth, int what, result_t res, int cacheit) {
 	      case AV_ATTRIB_TYPE:
 		sv = newSViv((int) curField->type);
 		break;
+	      case AV_ATTRIB_SQL_TYPE:
+		{   int i;
+#ifdef DBD_MYSQL
+		    switch (curField->type) {
+		      case FIELD_TYPE_DECIMAL:     i = SQL_DECIMAL;     break;
+		      case FIELD_TYPE_TINY:        i = SQL_TINYINT;     break;
+		      case FIELD_TYPE_SHORT:       i = SQL_SMALLINT;    break;
+		      case FIELD_TYPE_LONG:        i = SQL_INTEGER;     break;
+		      case FIELD_TYPE_FLOAT:       i = SQL_REAL;        break;
+		      case FIELD_TYPE_DOUBLE:      i = SQL_DOUBLE;      break;
+		      case FIELD_TYPE_TIMESTAMP:   i = SQL_TIMESTAMP;   break;
+		      case FIELD_TYPE_LONGLONG:    i = SQL_BIGINT;      break;
+		      case FIELD_TYPE_INT24:       i = SQL_BIGINT;      break;
+		      case FIELD_TYPE_DATE:        i = SQL_DATE;        break;
+		      case FIELD_TYPE_TIME:        i = SQL_TIME;        break;
+		      case FIELD_TYPE_NEWDATE:     i = SQL_DATE;        break;
+		      case FIELD_TYPE_TINY_BLOB:   i = SQL_VARCHAR;     break;
+		      case FIELD_TYPE_MEDIUM_BLOB: i = SQL_LONGVARCHAR; break;
+		      case FIELD_TYPE_LONG_BLOB:   i = SQL_LONGVARCHAR; break;
+		      case FIELD_TYPE_BLOB:        i = SQL_LONGVARCHAR; break;
+		      case FIELD_TYPE_VAR_STRING:  i = SQL_VARCHAR;     break;
+		      case FIELD_TYPE_STRING:      i = SQL_CHAR;        break;
+		      case FIELD_TYPE_DATETIME:    i = SQL_TIMESTAMP;   break;
+		      case FIELD_TYPE_NULL:        i = SQL_VARCHAR;     break;
+		      case FIELD_TYPE_YEAR:        i = SQL_SMALLINT;    break;
+		      case FIELD_TYPE_ENUM:        i = SQL_CHAR;        break;
+		      case FIELD_TYPE_SET:         i = SQL_CHAR;        break;
+		      default:                     i = SQL_VARCHAR;     break;
+		    }
+#else
+		    switch (curField->type) {
+		        case INT_TYPE:    i = SQL_INTEGER;  break;
+		        case CHAR_TYPE:   i = SQL_VARCHAR;  break;
+		        case REAL_TYPE:   i = SQL_REAL;     break;
+		        case IDENT_TYPE:  i = SQL_VARCHAR;  break;
+		        case NULL_TYPE:   i = SQL_VARCHAR;  break;
+		        case TEXT_TYPE:   i = SQL_VARCHAR;  break;
+		        case DATE_TYPE:   i = SQL_DATE;     break;
+		        case UINT_TYPE:   i = SQL_BIGINT;   break;
+		        case MONEY_TYPE:  i = SQL_VARCHAR;  break;
+		        case TIME_TYPE:   i = SQL_TIME;     break;
+		        case IDX_TYPE:    i = SQL_VARCHAR;  break;
+		        case SYSVAR_TYPE: i = SQL_VARCHAR;  break;
+		        case ANY_TYPE:    i = SQL_VARCHAR;  break;
+		        default:          i = SQL_VARCHAR;  break;
+		    }
+#endif
+		    sv = newSViv(i);
+		}
+		break;
 	      case AV_ATTRIB_IS_PRI_KEY:
 		sv = boolSV(IS_PRI_KEY(curField->flags));
 		break;
@@ -1400,14 +1465,16 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
 	}
 	break;
       case 'T':
-	/*
-	 *  Deprecated, use lower case versions.
-	 */
-	if (strEQ(key, "TABLE")) {
-	    retsv = ST_FETCH_AV(AV_ATTRIB_TABLE);
-	} else if (strEQ(key, "TYPE")) {
-	    retsv = ST_FETCH_AV(AV_ATTRIB_TYPE);
+	if (strEQ(key, "TYPE")) {
+	    retsv = ST_FETCH_AV(AV_ATTRIB_SQL_TYPE);
 	}
+	/*
+	 *  Deprecated, use lower case version.
+	 */
+	else if (strEQ(key, "TABLE")) {
+	    retsv = ST_FETCH_AV(AV_ATTRIB_TABLE);
+	}
+	break;
       case 'f':
 	if (strEQ(key, "format_max_size")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_LENGTH);
@@ -1443,15 +1510,21 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_LENGTH);
 	}
 	break;
-#if defined(DBD_MYSQL)
       case 'm':
+#if defined(DBD_MYSQL)
 	if (strEQ(key, "max_length")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_MAX_LENGTH);
 	} else if (strEQ(key, "mysql_use_result")) {
 	    retsv = boolSV(imp_sth->use_mysql_use_result);
+	} else if (strEQ(key, "mysql_type")) {
+	    retsv = ST_FETCH_AV(AV_ATTRIB_TYPE);
+	}
+#else
+	if (strEQ(key, "msql_type")) {
+	    retsv = ST_FETCH_AV(AV_ATTRIB_TYPE);
 	}
-	break;
 #endif
+	break;
       case 'r':
 	if (strEQ(key, "result")) {
 	    retsv = sv_2mortal(newSViv((IV) imp_sth->cda));
@@ -1460,9 +1533,8 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
       case 't':
 	if (strEQ(key, "table")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_TABLE);
-	} else if (strEQ(key, "type")) {
-	    retsv = ST_FETCH_AV(AV_ATTRIB_TYPE);
 	}
+	break;
     }
 
     if (dbis->debug >= 2) {
