@@ -66,6 +66,7 @@ enum av_attribs {
     AV_ATTRIB_NAME = 0,
     AV_ATTRIB_TABLE,
     AV_ATTRIB_TYPE,
+    AV_ATTRIB_SQL_TYPE,
     AV_ATTRIB_IS_PRI_KEY,
     AV_ATTRIB_IS_NOT_NULL,
     AV_ATTRIB_NULLABLE,
@@ -115,6 +116,7 @@ struct imp_dbh_st {
     dbh_t svsock;           /*  socket number for msql, &mysql for
 			     *  mysql
 			     */
+    int compression;        /*  Using compression.  */
 };
 
 
